@@ -7,7 +7,6 @@ from django.db.models import Manager, ForeignKey, OneToOneField, options
 from django.db.models.base import ModelBase
 from django.db.models.signals import post_init
 from django.utils.functional import cached_property
-from six import with_metaclass
 
 from modeltranslation import settings as mt_settings
 from modeltranslation.fields import (
@@ -52,7 +51,7 @@ class FieldsAggregationMetaClass(type):
         return super(FieldsAggregationMetaClass, cls).__new__(cls, name, bases, attrs)
 
 
-class TranslationOptions(with_metaclass(FieldsAggregationMetaClass, object)):
+class TranslationOptions(metaclass=FieldsAggregationMetaClass):
     """
     Translatable fields are declared by registering a model using
     ``TranslationOptions`` class with appropriate ``fields`` attribute.
@@ -187,13 +186,6 @@ def add_translation_fields(model, opts):
         model._meta.get_fields()
 
 
-def has_custom_queryset(manager):
-    "Check whether manager (or its parents) has declared some custom get_queryset method."
-    old_diff = getattr(manager, 'get_query_set', None) != getattr(Manager, 'get_query_set', None)
-    new_diff = getattr(manager, 'get_queryset', None) != getattr(Manager, 'get_queryset', None)
-    return old_diff or new_diff
-
-
 def patch_manager_class(manager):
     if isinstance(manager, MultilingualManager):
         return
@@ -639,7 +631,7 @@ class Translator(object):
 
         return self._registry[model]
 
-    def get_options_for_model(self, model):
+    def get_options_for_model(self, model) -> TranslationOptions:
         """
         Thin wrapper around ``_get_options_for_model`` to preserve the
         semantic of throwing exception for models not directly registered.
